/*
    pwl9x "Windows 9x PassWord List reader"
    Copyright (C) 2001 Knispel Guillaume

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "defs.h"
#include "x86.h"

void try_crack_pwl_x86(Params *p, Pwl *pwl)
{
  int i, k, n; FILE *alpha; x86workspace *buff8al; bool first;
  char buff[sizeof(x86workspace)+8], tmpinc[256];
  
  buff8al = (x86workspace *)((((unsigned int)buff) + 7) & ~7);

  printf("Crack\n---\n\n");

  if (!p->have_alphabet) { fprintf(stderr, "The only method implemented right now is brute forcing, please provide an alphabet.\n"); exit(EMET); }

  for (i = 0; i < 256; i++) tmpinc[i]  = 0;
  if (!(alpha = fopen(p->abc_file, "ro"))) { fprintf(stderr, "Bad alphabet file name.\n"); exit(EMET); }
  while ((i = fgetc(alpha))!=EOF) {
    if (i != '\n') tmpinc[i] = 1;
  }
  fclose(alpha);

  if (p->have_pwd) {
    for(i=0; p->pwd[i]; i++) if (!tmpinc[(byte)p->pwd[i]]) { fprintf(stderr, "Initial password seed contains invalid char '%c'.\n", p->pwd[i]); exit(EPWD); }
    n = strlen(p->pwd);
  } else n = 0;

  if (n > p->maxpwdlen) { fprintf(stderr, "Warning : start password longer than max password lenght, assume it's the correct one.\n"); return; }

  first = true;
  do {
    printf("Trying passwords of lenght %d\n", n);
    memset(buff8al, 0, sizeof(x86workspace));
    strcpy(buff8al->name, p->username); buff8al->salt = pwl->head->salt[0x10];
    memcpy(buff8al->sign, pwl->head->sign, sizeof(hres) * 2);
    for(k = i = 0; i < 256; i++) if (tmpinc[i]) { buff8al->incr[k] = i; k = i; }; buff8al->incr[k] = buff8al->incr[0];
    if (first) { first = false; if (p->have_pwd) strcpy(buff8al->pwd, p->pwd); } 
    else { for (i = 0; i < n; i++) buff8al->pwd[i] = buff8al->incr[0]; }
    buff8al->pwd[n] = 0;
    n++;
  } while(!(p->have_pwd = x86crk(buff8al)) && (n <= p->maxpwdlen));

  if (p->have_pwd) {
    if (!(p->pwd = malloc(n--))) { fprintf(stderr, "Could not allocate enough memory.\n"); exit(EMEM); }

    strcpy(p->pwd, buff8al->x86spc);
    k = buff8al->alq;

    if (n) while (k--) {
      char *linc = buff8al->incr; int r, s = 0;
      do {
	r = p->pwd[s];
	for (i = 1; (i < 256) && (p->pwd[s] != linc[i]); i++); p->pwd[s] = i;
	s++;
      } while (i >= r && s < n);
    }

    printf("The main password seems to be \"%s\"\n\n", p->pwd);
  } else printf("The main password has not been found\n");
}
